package com.sun.media.sound;

import java.io.IOException;
import java.util.Vector;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFormat;

public final class UlawCodec extends SunCodec
{
    private static final byte[] ULAW_TABH;
    private static final byte[] ULAW_TABL;
    private static final AudioFormat.Encoding[] ulawEncodings;
    private static final short[] seg_end;
    
    public UlawCodec() {
        super(UlawCodec.ulawEncodings, UlawCodec.ulawEncodings);
    }
    
    @Override
    public AudioFormat.Encoding[] getTargetEncodings(AudioFormat audioFormat) {
        if (AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding())) {
            if (audioFormat.getSampleSizeInBits() == 16) {
                return new AudioFormat.Encoding[] { AudioFormat.Encoding.ULAW };
            }
            return new AudioFormat.Encoding[0];
        }
        else if (AudioFormat.Encoding.ULAW.equals(audioFormat.getEncoding())) {
            if (audioFormat.getSampleSizeInBits() == 8) {
                return new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED };
            }
            return new AudioFormat.Encoding[0];
        }
        else {
            return new AudioFormat.Encoding[0];
        }
    }
    
    @Override
    public AudioFormat[] getTargetFormats(AudioFormat.Encoding encoding, AudioFormat audioFormat) {
        if ((AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && AudioFormat.Encoding.ULAW.equals(audioFormat.getEncoding())) || (AudioFormat.Encoding.ULAW.equals(encoding) && AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding()))) {
            return this.getOutputFormats(audioFormat);
        }
        return new AudioFormat[0];
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat.Encoding encoding, AudioInputStream audioInputStream) {
        AudioFormat format = audioInputStream.getFormat();
        AudioFormat.Encoding encoding2 = format.getEncoding();
        if (encoding2.equals(encoding)) {
            return audioInputStream;
        }
        if (!this.isConversionSupported(encoding, audioInputStream.getFormat())) {
            throw new IllegalArgumentException("Unsupported conversion: " + audioInputStream.getFormat().toString() + " to " + encoding.toString());
        }
        AudioFormat audioFormat;
        if (AudioFormat.Encoding.ULAW.equals(encoding2) && AudioFormat.Encoding.PCM_SIGNED.equals(encoding)) {
            audioFormat = new AudioFormat(encoding, format.getSampleRate(), 16, format.getChannels(), 2 * format.getChannels(), format.getSampleRate(), format.isBigEndian());
        }
        else if (AudioFormat.Encoding.PCM_SIGNED.equals(encoding2) && AudioFormat.Encoding.ULAW.equals(encoding)) {
            audioFormat = new AudioFormat(encoding, format.getSampleRate(), 8, format.getChannels(), format.getChannels(), format.getSampleRate(), false);
        }
        else {
            throw new IllegalArgumentException("Unsupported conversion: " + audioInputStream.getFormat().toString() + " to " + encoding.toString());
        }
        return this.getAudioInputStream(audioFormat, audioInputStream);
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat audioFormat, AudioInputStream audioInputStream) {
        return this.getConvertedStream(audioFormat, audioInputStream);
    }
    
    private AudioInputStream getConvertedStream(AudioFormat format, AudioInputStream audioInputStream) {
        if (audioInputStream.getFormat().matches(format)) {
            return audioInputStream;
        }
        return new UlawCodec.UlawCodecStream(audioInputStream, format);
    }
    
    private AudioFormat[] getOutputFormats(AudioFormat audioFormat) {
        Vector<AudioFormat> vector = new Vector();
        if (audioFormat.getSampleSizeInBits() == 16 && AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding())) {
            vector.addElement(new AudioFormat(AudioFormat.Encoding.ULAW, audioFormat.getSampleRate(), 8, audioFormat.getChannels(), audioFormat.getChannels(), audioFormat.getSampleRate(), false));
        }
        if (AudioFormat.Encoding.ULAW.equals(audioFormat.getEncoding())) {
            vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), 16, audioFormat.getChannels(), audioFormat.getChannels() * 2, audioFormat.getSampleRate(), false));
            vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), 16, audioFormat.getChannels(), audioFormat.getChannels() * 2, audioFormat.getSampleRate(), true));
        }
        AudioFormat[] array = new AudioFormat[vector.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (AudioFormat)(AudioFormat)vector.elementAt(i);
        }
        return array;
    }
    
    static {
        ULAW_TABH = new byte[256];
        ULAW_TABL = new byte[256];
        ulawEncodings = new AudioFormat.Encoding[] { AudioFormat.Encoding.ULAW, AudioFormat.Encoding.PCM_SIGNED };
        seg_end = new short[] { 255, 511, 1023, 2047, 4095, 8191, 16383, 32767 };
        for (int i = 0; i < 256; ++i) {
            int n = ~i & 0xFF;
            int n2 = ((n & 0xF) << 3) + 132 << ((n & 0x70) >> 4);
            int n3 = ((n & 0x80) != 0x0) ? (132 - n2) : (n2 - 132);
            UlawCodec.ULAW_TABL[i] = (byte)(n3 & 0xFF);
            UlawCodec.ULAW_TABH[i] = (byte)(n3 >> 8 & 0xFF);
        }
    }
    
    class UlawCodecStream extends AudioInputStream
    {
        private static final int tempBufferSize = 64;
        private byte[] tempBuffer;
        boolean encode;
        AudioFormat encodeFormat;
        AudioFormat decodeFormat;
        byte[] tabByte1;
        byte[] tabByte2;
        int highByte;
        int lowByte;
        
        UlawCodecStream(AudioInputStream stream, AudioFormat audioFormat) {
            super(stream, audioFormat, -1L);
            this.tempBuffer = null;
            this.encode = false;
            this.tabByte1 = null;
            this.tabByte2 = null;
            this.highByte = 0;
            this.lowByte = 1;
            AudioFormat format = stream.getFormat();
            if (!UlawCodec.this.isConversionSupported(audioFormat, format)) {
                throw new IllegalArgumentException("Unsupported conversion: " + format.toString() + " to " + audioFormat.toString());
            }
            int n;
            if (AudioFormat.Encoding.ULAW.equals(format.getEncoding())) {
                this.encode = false;
                this.encodeFormat = format;
                this.decodeFormat = audioFormat;
                n = audioFormat.isBigEndian();
            }
            else {
                this.encode = true;
                this.encodeFormat = audioFormat;
                this.decodeFormat = format;
                n = format.isBigEndian();
                this.tempBuffer = new byte[64];
            }
            if (!(n == 0)) {
                this.tabByte1 = UlawCodec.ULAW_TABH;
                this.tabByte2 = UlawCodec.ULAW_TABL;
                this.highByte = 0;
                this.lowByte = 1;
            }
            else {
                this.tabByte1 = UlawCodec.ULAW_TABL;
                this.tabByte2 = UlawCodec.ULAW_TABH;
                this.highByte = 1;
                this.lowByte = 0;
            }
            if ((stream instanceof AudioInputStream)) {
                this.frameLength = stream.getFrameLength();
            }
            this.framePos = 0L;
            this.frameSize = format.getFrameSize();
            if (this.frameSize == -1) {
                this.frameSize = 1;
            }
            return;
        }
        
        private short search(short n, short[] array, short n2) {
            int i = 0;
            while (i < n2) {
                if (n <= array[i]) {
                    return i;
                }
                i = (short)(i + 1);
            }
            return n2;
        }
        
        @Override
        public int read() throws IOException {
            byte[] array = { 0 };
            if (this.read(array, 0, array.length) == 1) {
                return array[1] & 0xFF;
            }
            return -1;
        }
        
        @Override
        public int read(byte[] array) throws IOException {
            return this.read(array, 0, array.length);
        }
        
        @Override
        public int read(byte[] b, int n, int n2) throws IOException {
            if (n2 % this.frameSize != 0) {
                n2 -= n2 % this.frameSize;
            }
            if (this.encode) {
                int n3 = 132;
                int len2;
                int read2;
                int j;
                int n4;
                int n5;
                int n6;
                int search;
                int n7;
                for (len2 = n2 * 2, len2 = ((len2 > 64) ? 64 : len2); (read2 = super.read(this.tempBuffer, 0, len2)) > 0; len2 -= read2, len2 = ((len2 > 64) ? 64 : len2)) {
                    for (j = 0; j < read2; b[n] = n7, ++n, j += 2) {
                        n4 = (short)((short)(this.tempBuffer[j + this.highByte] << 8 & 0xFF00) | (short)((short)this.tempBuffer[j + this.lowByte] & 0xFF));
                        if (n4 < 0) {
                            n5 = (short)(n3 - n4);
                            n6 = 127;
                        }
                        else {
                            n5 = (short)(n4 + n3);
                            n6 = 255;
                        }
                        search = this.search((short)n5, UlawCodec.seg_end, (short)8);
                        if (search >= 8) {
                            n7 = (byte)(0x7F ^ n6);
                        }
                        else {
                            n7 = (byte)((byte)(search << 4 | (n5 >> search + 3 & 0xF)) ^ n6);
                        }
                    }
                }
                if (n == n && read2 < 0) {
                    return read2;
                }
                return n - n;
            }
            else {
                int len = n2 / 2;
                int off = n + n2 / 2;
                int read = super.read(b, off, len);
                if (read < 0) {
                    return read;
                }
                int i;
                for (i = n; i < n + read * 2; i += 2) {
                    b[i] = this.tabByte1[b[off] & 0xFF];
                    b[i + 1] = this.tabByte2[b[off] & 0xFF];
                    ++off;
                }
                return i - n;
            }
        }
    }
}
